//file higgins.h

extern void dacon(double *, double *);
extern void model(int, int, double *, double *, double *);
extern void pend(double, double, double, int, int);
extern void dose(double *);
extern void lamset(int, int);

enum {LUNGS = 0, PERIPHERAL_SHUNT = 1, KIDNEYS = 2, GUT_SPLEEN = 3,
      LIVER = 4, OTHER_VISCERA = 5, MUSCLE = 6, FAT = 7, BRAIN = 8,
      IM_INJECTION_SITE = 9, NASAL_MUCOSAL_SITE = 10 };

enum {ARTERIAL_POOL = 0, PORTAL_POOL = 1, MUSCLE_POOL = 2,
      FAT_POOL = 3, VENOUS_POOL = 4,
      INJECTION_POOL = 5, PERIPHERAL_S_POOL = 6,
      FIRST_VENOUS_MIX = 7, SECOND_VENOUS_MIX = 8,
      THIRD_VENOUS_MIX = 9, FOURTH_VENOUS_MIX = 10,
      FIFTH_VENOUS_MIX = 11, SIXTH_VENOUS_MIX = 12,
      SEVENTH_VENOUS_MIX = 13 };

#define NR_END 1
#define FREE_ARG char*
#define FALSE 0
#define TRUE (!FALSE)
#define ESC '\x1B'


////////////////////////////////////////////////
//                                            //
//  Declaration of drug-dependent variables   //
//                                            //
////////////////////////////////////////////////

	int nmber_compt, rdnmber_compt, nmber_prmchng, protbind_exp;
	int nmb_permixes;
	double acidity;

	double pKa, lipwat_part, urine_pH, livmetex_fr;
	double pend_ivdose, pend_imdose, pend_ipdose = 0.0, pend_pHplas;
	double pend_cycle, pend_time;

//////////////////////////////////////////////////
//                                              //
//  Declaration of patient-dependent variables  //
//                                              //
//////////////////////////////////////////////////

	double body_weight, bldtotmass_fr, celbld_fr;
	double pend_cardop, pend_infdur;
	double *celtis_fr;

////////////////////////////////////////////////
//                                            //
//  Declaration of model-dependent variables  //
//                                            //
////////////////////////////////////////////////

	double artpool_bfr, portpool_bfr, musclpool_bfr;
	double fatpool_bfr, venpool_bfr, injpool_bfr;
	double watplas_fr, protplas_fr;
	double sim_time;
	double *wattis_fr, *liptis_fr, *prottis_fr;
	double *tistotmass_fr, *watecf_fr;
	double *lipecf_fr, *protecf_fr, *cardop_fr;
	double *negslope_pHt, *negoffset_pHt;
	double plasprot_mlms, ecfprot_mlms;
	double *celprot_mlms;


/////////////////////////////////////////
//                                     //
// Declaration of program variables    //
//                                     //
/////////////////////////////////////////

	int j_loop = 0;
	double *stanconc_old, *conc_higgins, *tis2comp_drgfr;
	double *tisdrg_begequ, *bldtisdrg_equ, *blddrg_begequ;
	double *tisdrg_equ, *compsvmass_fr, *est_stconc;

	double *bld_conc, *plas_conc;
	double *bld_concv, *plas_concv;
	double *tiscompmass_fr, cycle_time = 0.0;
	double pershunt_drg, pershunt_mass;

	double artpool_mass, injpool_mass, venpool_mass;
	double portpool_mass, fatpool_mass, musclpool_mass;
	double artpool_drg = 0.0, portpool_drg = 0.0, injpool_drg = 0.0;
	double fatpool_drg = 0.0, musclpool_drg = 0.0;
	double venpool_drg = 0.0, run_time = 0.0;
	double dose_increment = 0.0, total_dose = 0.0, iv_dose = 0.0;
	double im_dose = 0.0, ip_dose = 0.0, cumul_met = 0.0, cumul_excr = 0.0;
	double pool_drug, tissue_drug = 0.0, temp_main;
	long cycle_number = 0;

///////////////////

	double *watfr_tiscelfr, *lipfr_tiscelfr, *protfr_tiscelfr;
	double blood_weight;

	double *pH_cel, *pH_ecf, pH_plas;
	double stroke_volweight, inf_duration;

	double muscl2pool_drgfr, fat2pool_drgfr;
	double ven2pool_drgfr, port2pool_drgfr;
	double vsvfr2pool_mass, msvfr2pool_mass, fsvfr2pool_mass;
	double psvfr2pool_mass;

//////////////////////////////////////
// Declaration of new variables     //
// unique to "fentanyl"             //
//////////////////////////////////////

	double urine_flow, urine_concfct, wt_muc;
	double protmuc_fr, watmuc_fr, nasal_pH;
	double mucprot_mlms;
	double erythplas_ratio, protbnd_cnstif, protbnd_cnstuf;

	double *x_higgins, *b_higgins, *c_higgins, *e_higgins;
	double aa_mucus = 0.0, aabt_mucus = 0.0;
	double R_plas, R_ecf, R_mucus, m_higgins, u_higgins;


double *vector(long nl, long nh)
{
	double *v;
	v = (double *)calloc((nh - nl + 1 + NR_END), sizeof(double));
	return v - nl + NR_END;
}


