#include <math.h>

#include "nlgpc.h"
#include "mex.h"

float ran2(long *idum)
{
   int jn;
   long kn;
   static long idum2=123456789;
   static long iy=0;
   static long iv[NTAB];
   float temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) *idum=1;
      else *idum = -(*idum);
      idum2=(*idum);
      for (jn=NTAB+7;jn>=0;jn--) {
         kn=(*idum)/IQ1;
         *idum=IA1*(*idum-kn*IQ1)-kn*IR1;
         if (*idum < 0) *idum += IM1;
         if (jn < NTAB) iv[jn] = *idum;
      }
      iy=iv[0];
   }
   kn=(*idum)/IQ1;
   *idum=IA1*(*idum-kn*IQ1)-kn*IR1;
   if (*idum < 0) *idum += IM1;
   kn=idum2/IQ2;
   idum2=IA2*(idum2-kn*IQ2)-kn*IR2;
   if (idum2 < 0) idum2 += IM2;
   jn=iy/NDIV;
   iy=iv[jn]-idum2;
   iv[jn] = *idum;
   if (iy < 1) iy += IMM1;
   if ((temp=AM*iy) > RNMX) return RNMX;
   else return temp;
}

double gasdev(idum)
int *idum;
{
   double fac,r,v1,v2;
   if  (iset == 0) {
      do {
         v1=2.0*ran2(idum)-1.0;
         v2=2.0*ran2(idum)-1.0;
         r=v1*v1+v2*v2;
      } while (r >= 1.0);
      fac=sqrt(-2.0*log(r)/r);
      gset=v1*fac;
      iset=1;
      return v2*fac;
   } else {
      iset=0;
      return gset;
   }
}

void derivs(double yyx[], double dyyx[])
{
   dyyx[ND-1] = u_out -den[1]*yyx[ND-1];
   for (i=0; i<ND-1; i++) {
      dyyx[i] = yyx[i+1];
      dyyx[ND-1] -= den[ND-i]*yyx[i];
   }
}

void rk4(double ry[], double h, double ryout [],
	 void(*derivs)(double [], double []))
{
   double hh, h6;

   hh = h * 0.50;
   h6 = h / 6;
   (*derivs)(ry, dydx);
   for (i = 0; i <= ND - 1; i++) yt[i] = ry[i] + hh * dydx[i];
   (*derivs)(yt, dyt);
   for (i = 0; i <= ND - 1; i++) yt[i] = ry[i] + hh * dyt[i];
   (*derivs)(yt, dym);
   for (i = 0; i <= ND - 1; i++) {
      yt[i] = ry[i] + h * dym[i];
      dym[i] += dyt[i];
   }
   (*derivs)(yt, dyt);
   for (i = 0; i <= ND - 1; i++) {
      ryout[i] = ry[i] + h6 * (dydx[i] + dyt[i] + 2 * dym[i]);
      ry[i] = ryout[i];
   }
}

void str_prm()
{
   for (i = 0; i <= na - 1; i++) a[i + 1] = prm[i]; 
   if (idelm > 0) for (i = 0; i <= idelm - 1; i++) b[i] = 0; 
   for (i = it - 1; i <= itb - 1; i++) b[i] = prm[na + i - idelm]; 
}

void solv_dioph()
{
   int ij;
   e[0] = (pn[0] / pd[0]) * t0;
   pt[0] = pn[0] * t0;
   alpha[0] = 1;
   alpa[0] = pd[0];

   for (i = 1; i <= npt - 1; i++) {
      pt[i] = 0;
      for (j = 0; j <= npdp - 1; j++) {
         if (i >= j) pt[i] += pn[j] * t[i-j];
         else  pt[i] = pt[i];
      }
   }
   for (i = 1; i <= napp - 1; i++) alpha[i] = a[i] - a[i - 1];
   for (i = 1; i <= nao - 1; i++) {
      alpa[i] = 0;
      for (j = 0; j <= npdp - 1; j++) {
         if (i >= j) alpa[i] += alpha[i-j] * pd[j];
         else alpa[i] = alpa[i];
      }
   }
   for (j = 0; j <= nptm - 1; j++) f[0][j] = pt[j + 1] - e[0] * alpa[j + 1];
   for (i = 1; i <= n2 - 1; i++) {
      e[i] = f[i - 1][0] / pd[0];
      for (j = 0; j <= nptm - 1; j++) f[i][j] = f[i - 1][j + 1] - e[i] * alpa[j + 1];
   }
}

void g_elements()
{
   g[0] = (1 / t0) * (e[0] * b[0]);
   for (j = 0; j <= itb - 1; j++) gamma[0][j] = e[0] * b[j+1] - g[0] * t[j+1];

   for (i = 1; i <= n2 - 1; i++) {
      g[i] = (1 / t0) * (e[i] * b[0] + gamma[i-1][0]);
      for (j = 0; j <= itb - 1; j++) {
         gamma[i][j] = gamma[i-1][j+1] + e[i] * b[j+1] - g[i] * t[j+1];
      }
   }
}

void vv_calc()
{
   double tempf = 0;
   int nmax;

   for (i = n1 - 1; i <= n2 - 1; i++) {
      tempf = 0;
      nmax=itb; if (itb<nptm) nmax=nptm;     
      for (j=0; j<=nmax-1; j++) tempf+=f[i][j]* yfdt[19-j]+gamma[i][j]*duft[18-j];
      vv[i] = tempf;
   }
}

double obj_func(double unknown)
{
   double tempf = 0, sumin_cost = 0;

   for (i = n1 - 1; i <= n2 - 1; i++) {
      tempf = vv[i] + (g[i] * unknown);
      omega=omega_0[i];
      if (tempf>0) tempf=(pow(tempf,mdl_pow))/(pow(tempf,mdl_pow)+pow(mdl_x50, mdl_pow));
      else tempf = 0;
      sumin_cost += pow((omega - tempf), 2);
   }
   sumin_cost += (tun * pow(unknown, 2));
   return(sumin_cost);
}

void mnbrak(double *ax, double *bx, double *cx, double *fa, double *fb, double *fc,
	double (*obj_func)(double))
{
   double ulim,u,r,q,fu,dum;

   *fa=(*obj_func)(*ax);
   *fb=(*obj_func)(*bx);
   if (*fb > *fa) {
      SHFT(dum,*ax,*bx,dum)
      SHFT(dum,*fb,*fa,dum)
   }
   *cx=(*bx)+GOLD*(*bx-*ax);
   *fc=(*obj_func)(*cx);
   while (*fb > *fc) {
      r=(*bx-*ax)*(*fb-*fc);
      q=(*bx-*cx)*(*fb-*fa);
      u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/
         (2*SIGN(FMAX(fabs(q-r),TINY),q-r));
      ulim=(*bx)+GLIMIT*(*cx-*bx);
      if ((*bx-u)*(u-*cx) > 0) {
         fu=(*obj_func)(u);
         if (fu < *fc) {
            *ax=(*bx); *bx=u; *fa=(*fb); *fb=fu; return;
         }
         else if (fu > *fb) {
            *cx=u; *fc=fu; return;
         }
         u=(*cx)+GOLD*(*cx-*bx);
         fu=(*obj_func)(u);
      }
      else if ((*cx-u)*(u-ulim) > 0) {
         fu=(*obj_func)(u);
         if (fu < *fc) {
            SHFT(*bx,*cx,u,*cx+GOLD*(*cx-*bx))
            SHFT(*fb,*fc,fu,(*obj_func)(u))
         }
      } 
      else if ((u-ulim)*(ulim-*cx) >= 0) {
         u=ulim;
         fu=(*obj_func)(u);
      }
      else {
         u=(*cx)+GOLD*(*cx-*bx);
         fu=(*obj_func)(u);
      }
      SHFT(*ax,*bx,*cx,u)
      SHFT(*fa,*fb,*fc,fu)
   }
}

double golden(double ax, double bx, double cx, double (*obj_func)(double), double tol,
	double *xmin)
{
   double f1,f2,x0,x1,x2,x3;

   x0=ax; x3=cx;
   if (fabs(cx-bx) > fabs(bx-ax)) {
      x1=bx;
      x2=bx+C*(cx-bx);
   }
   else {
      x2=bx;
      x1=bx-C*(bx-ax);
   }
   f1=(*obj_func)(x1);
   f2=(*obj_func)(x2);
   while (fabs(x3-x0) > tol*(fabs(x1)+fabs(x2))) {
      if (f2 < f1) {
         SHFT(x0,x1,x2,R*x1+C*x3)
         SHFT2(f1,f2,(*obj_func)(x2))
      }
      else {
         SHFT(x3,x2,x1,R*x2+C*x0)
         SHFT2(f2,f1,(*obj_func)(x1))
      }
   }
   if (f1 < f2) {
      *xmin=x1; return f1;
   }
   else {
      *xmin=x2;  return f2;
   }
}

void mea_vec(double *yg, double *ug)
{
   for (i = 0; i <= na - 1; i++) phi[i] = - yg[18-i];
   for (i = na; i <= na + nb + iov - 1; i++) phi[i] = ug[18+na-i-idelm];
}

void udf(double pe, double *phi, double *diag, double *kalgain, double rho,
	 double *cov, int nay, double prm[])
{
   int itt = -1, iww = -1, Jm;
   double ar, av, as, aal, ap, az;

   ar = phi[0];
   av = diag[0] * ar;
   kalgain[0] = av;
   as = 1 + av * ar;
   diag[0] = diag[0] / (as * rho);
   for (j = 1; j <= nay - 1; j++) {
      ar = phi[j];
      Jm = j - 1;
      for (i = 0; i <= Jm; i++) {
         itt++;
         ar += phi[i] * cov[itt];
      }
      av = ar * diag[j];
      kalgain[j] = av;
      aal = as;
      as = aal + av * ar;
      diag[j] *= aal / (as * rho);
      ap = - ar / aal;
      for (i = 0; i <= Jm; i++) {
         iww++;
         az = cov[iww] + kalgain[i] * ap;
         kalgain[i] += cov[iww] * av;
         cov[iww] = az;
      }
   }
   for (i = 0; i <= nay - 1; i++) prm[i] += (pe * kalgain[i] / as);
}

void shift_data()
{
   for (i = 1; i <= 19; i++) {
      y[i-1] = y[i];
      u[i-1] = u[i];
      uf[i-1] = uf[i];
      yf[i-1] = yf[i];
      duft[i-1] = duft[i];
      duft_rls[i-1] = duft_rls[i];
      yd[i-1] = yd[i];
      yfdt[i-1] = yfdt[i];
      dyft[i-1] = dyft[i];
      dyft_rls[i-1] = dyft_rls[i];
      omega_0[i-1]=omega_0[i];
   }
}

void read_parameters()
{
   int i;
   nmber_compt = 11;      /* Number of Compartments */
   protbind_exp = 1;      /* protein Binding Expression */
           /* "0": binding of unioneised drug only to protein molecules */
           /* "1": binding of all forms of drug to protein molecules */
   nmb_permixes = 14;     /* Number of calculated Peripheral Mixes */
   acidity = 1.0;         /* Acid or base drug indicator: */
                     /* "1": for basic drug, "-1": for acidic drug */
   pKa = 7.85;               /* fentanyl pKa */
   lipwat_part = 500.0;        /* Lipid-Water partition coefficient */
   urine_pH = 6.2;           /* Urine pH */
   livmetex_fr = 0.55;     /* Liver metabolic extraction fraction */
   pend_ivdose = 36.0;    /* Pending intra-venous drug dose in ugs */
   body_weight = 70.001;    /* Total body weight in Kgs */
   bldtotmass_fr = 0.07857; /* Kg blood / Kg total body mass */
   celbld_fr = 0.45; /* Haematocrit: g cells / g blood; */
                            /* volume of red cells in blood */
   pend_pHplas = 7.40; /* Pending Plasma pH */
   pend_cardop = 6.48; /* Pending cardiac output: litres/min. */
   pend_infdur = 60.0; /* Pending infusion duration in seconds */

   for (i = 0; i <= 6; i++) celtis_fr[i] = 0.80;
   celtis_fr[FAT] = 0.925;       /* Kg cells / Kg of tissue */
   celtis_fr[BRAIN] = 0.70;
   celtis_fr[NASAL_MUCOSAL_SITE] = celtis_fr[IM_INJECTION_SITE] = 0.80;

   artpool_bfr = 0.1300;  /* Fraction of total blood volume: arterial pool */
   fatpool_bfr = 0.1010;   /* Fraction of total blood volume: fat pool */
   injpool_bfr = 0.1300;   /* Fraction of total blood volume: injection-site pool */
   musclpool_bfr = 0.3380; /* Fraction of total blood volume: muscle pool */
   portpool_bfr = 0.1710; /* Fraction of total blood volume: portal pool */
   venpool_bfr = 0.1300;   /* Fraction of total blood volume: venous pool */

   watplas_fr = 0.9155;    /* Kg water / Kg plasma */
   protplas_fr = 0.0680;   /* Kg protein / Kg plasma */
   sim_time = 21600.0;     /* Simulation time in seconds */

   wattis_fr[LUNGS] = 0.7500;
   wattis_fr[PERIPHERAL_SHUNT] = 0.00;
   wattis_fr[KIDNEYS] = 0.7700;
   wattis_fr[GUT_SPLEEN] = 0.7840;
   wattis_fr[LIVER] = 0.7200;
   wattis_fr[OTHER_VISCERA] = 0.7740;  /* Kg water / Kg tissue */
   wattis_fr[MUSCLE] = 0.7470;
   wattis_fr[FAT] = 0.1490;
   wattis_fr[BRAIN] = 0.7900;
   wattis_fr[IM_INJECTION_SITE] = 0.7900;
   wattis_fr[NASAL_MUCOSAL_SITE] = 0.7917;

   liptis_fr[LUNGS] = 0.0120;
   liptis_fr[PERIPHERAL_SHUNT] = 0.00;
   liptis_fr[KIDNEYS] = 0.0520;
   liptis_fr[GUT_SPLEEN] = 0.0570;
   liptis_fr[LIVER] = 0.0670;
   liptis_fr[OTHER_VISCERA] = 0.1050;  /* Kg lipid / Kg tissue */
   liptis_fr[MUSCLE] = 0.0450;
   liptis_fr[FAT] = 0.8000;
   liptis_fr[BRAIN] = 0.1100;
   liptis_fr[IM_INJECTION_SITE] = 0.0200;
   liptis_fr[NASAL_MUCOSAL_SITE] = 0.0617;

   prottis_fr[LUNGS] = 0.1800;
   prottis_fr[PERIPHERAL_SHUNT] = 0.00;
   prottis_fr[KIDNEYS] = 0.1700;
   prottis_fr[GUT_SPLEEN] = 0.1410;
   prottis_fr[LIVER] = 0.1800;
   prottis_fr[OTHER_VISCERA] = 0.0970;  /* Kg protein / Kg tissue */
   prottis_fr[MUSCLE] = 0.1910;
   prottis_fr[FAT] = 0.0490;
   prottis_fr[BRAIN] = 0.0800;
   prottis_fr[IM_INJECTION_SITE] = 0.1700;
   prottis_fr[NASAL_MUCOSAL_SITE] = 0.1333;

   tistotmass_fr[LUNGS] = 0.006628;
   tistotmass_fr[PERIPHERAL_SHUNT] = 0.00;
   tistotmass_fr[KIDNEYS] = 0.004057;
   tistotmass_fr[GUT_SPLEEN] = 0.0198700;
   tistotmass_fr[LIVER] = 0.02459;
   tistotmass_fr[OTHER_VISCERA] = 0.02686; /* Kg tissue / Kg total mass */
   tistotmass_fr[MUSCLE] = 0.52446;
   tistotmass_fr[FAT] = 0.19557;
   tistotmass_fr[BRAIN] = 0.00010;
   tistotmass_fr[IM_INJECTION_SITE] = 0.00014;
   tistotmass_fr[NASAL_MUCOSAL_SITE] = 0.0001186;

   watecf_fr[LUNGS] = 0.9650;
   watecf_fr[PERIPHERAL_SHUNT] = 0.9650;
   watecf_fr[KIDNEYS] = 0.9650;
   watecf_fr[GUT_SPLEEN] = 0.9650;
   watecf_fr[LIVER] = 0.9650;
   watecf_fr[OTHER_VISCERA] = 0.9650;  /* Kg water / Kg ecf */
   watecf_fr[MUSCLE] = 0.9650;
   watecf_fr[FAT] = 0.9650;
   watecf_fr[BRAIN] = 0.9900;
   watecf_fr[IM_INJECTION_SITE] = 0.9650;
   watecf_fr[NASAL_MUCOSAL_SITE] = 0.9650;

   for (i = 0; i <= 10; i++) lipecf_fr[i] = 0.0;   /* Kg lipid / Kg ecf */

   protecf_fr[LUNGS] = 0.0350;
   protecf_fr[PERIPHERAL_SHUNT] = 0.0350;
   protecf_fr[KIDNEYS] = 0.0350;
   protecf_fr[GUT_SPLEEN] = 0.0350;
   protecf_fr[LIVER] = 0.0350;
   protecf_fr[OTHER_VISCERA] = 0.0350;  /* Kg protein / Kg ecf */
   protecf_fr[MUSCLE] = 0.0350;
   protecf_fr[FAT] = 0.0350;
   protecf_fr[BRAIN] = 0.0100;
   protecf_fr[IM_INJECTION_SITE] = 0.0350;
   protecf_fr[NASAL_MUCOSAL_SITE] = 0.0350;

   cardop_fr[LUNGS] = 1.0;
   cardop_fr[PERIPHERAL_SHUNT] = 0.151 * (1.0 / 1.00162);
   cardop_fr[KIDNEYS] = 0.188 * (1.0 / 1.00162);
   cardop_fr[GUT_SPLEEN] = 0.171 * (1.0 / 1.00162);
   cardop_fr[LIVER] = 0.0690 * (1.0 / 1.00162);
   cardop_fr[OTHER_VISCERA] = 0.191 * (1.0 / 1.00162);
   cardop_fr[MUSCLE] = 0.17690 * (1.0 / 1.00162);        /* Fractional stroke volumes */
   cardop_fr[FAT] = 0.05300 * (1.0 / 1.00162);           /* correction factor to make */
   cardop_fr[BRAIN] = 0.00105 * (1.0 / 1.00162);         /* sum = 1.0 */
   cardop_fr[IM_INJECTION_SITE] = 0.00003 * (1.0 / 1.00162);
   cardop_fr[NASAL_MUCOSAL_SITE] = 0.000640 * (1.0 / 1.00162);

   negoffset_pHt[LUNGS] = -0.40;
   negoffset_pHt[PERIPHERAL_SHUNT] = 0.00;
   negoffset_pHt[KIDNEYS] = -0.30;
   negoffset_pHt[GUT_SPLEEN] = -0.60;
   negoffset_pHt[LIVER] = -0.30;
   negoffset_pHt[OTHER_VISCERA] = -0.60;  /* Negative offset pH equation for */
   negoffset_pHt[MUSCLE] = -0.46;         /* compartment tissues */
   negoffset_pHt[FAT] = -0.20;
   negoffset_pHt[BRAIN] = -0.30;
   negoffset_pHt[IM_INJECTION_SITE] = -0.46;
   negoffset_pHt[NASAL_MUCOSAL_SITE] = -0.30;

   negslope_pHt[LUNGS] = -0.80;
   negslope_pHt[PERIPHERAL_SHUNT] = 0.00;
   negslope_pHt[KIDNEYS] = -0.80;
   negslope_pHt[GUT_SPLEEN] = -0.80;
   negslope_pHt[LIVER] = -0.80;
   negslope_pHt[OTHER_VISCERA] = -0.80;  /* Negative slope pH equation for */
   negslope_pHt[MUSCLE] = -0.80;         /* for compartment tissue */
   negslope_pHt[FAT] = -0.80;
   negslope_pHt[BRAIN] = -0.80;
   negslope_pHt[IM_INJECTION_SITE] = -0.80;
   negslope_pHt[NASAL_MUCOSAL_SITE] = -0.80;

   pend_cycle = 1.0;       /* Pending cycle-time in seconds */
   plasprot_mlms = 91000.0; /* Mean molar mass of plasma proteins in g */
   ecfprot_mlms = 91000.0;   /* Mean molar mass of ecf proteins in g */

   celprot_mlms[LUNGS] = 91000.0;
   celprot_mlms[PERIPHERAL_SHUNT] = 0.0;
   celprot_mlms[KIDNEYS] = 91000.0;
   celprot_mlms[GUT_SPLEEN] = 91000.0;
   celprot_mlms[LIVER] = 91000.0;
   celprot_mlms[OTHER_VISCERA] = 91000.0;
   celprot_mlms[MUSCLE] = 91000.0;  /* Mean molar mass of cell proteins */
   celprot_mlms[FAT] = 91000.0;     /* in g */
   celprot_mlms[BRAIN] = 91000.0;
   celprot_mlms[IM_INJECTION_SITE] = 91000.0;
   celprot_mlms[NASAL_MUCOSAL_SITE] = 91000.0;

//
// Initialisation of the variables which are unique to "fentanyl"
//
   urine_flow = 100.0;         /* urine water output g/h */
   urine_concfct = 17.0;     /* urine concentration factor */
   wt_muc = 4.0 / 10000.0;   /* Kg mucous in nasal absorption site */
   protmuc_fr = 6.77 / 1000.0;   /* Kg protein / Kg mucous */
   watmuc_fr = 0.94;    /* Kg water / Kg mucous */
   nasal_pH = 6.0;     /* pH nasal secretions */
   mucprot_mlms = 91000.0;   /* average molecular weight for mucous proteins */
   erythplas_ratio = 0.908;   /* erythrocyte-plasma ratio */
              /* umol per Kg eryth / umol drug per Kg plasma */
   protbnd_cnstif = 0.001246;  /* protein binding constant ionised form */
   protbnd_cnstuf = 0.019121;  /* protein binding constant unionised form */

/**************************************************************************/
/*     Also hard-wired in the program are:                                */
/*      - the density of blood: 1.06 g/ml                                 */
/*      - the density of plasma: 1.03 g/ml                                */
/*      - the density of erythrocytes: 1.10 g/ml                          */
/*      - conversion coefficient from "ug" to "umoles": 1.0 / 336.60      */
/**************************************************************************/
   dose_increment = 0;
   rdnmber_compt = nmber_compt;
}

void lamset(int rdnmber_compt, int protbind_exp)
{
   int i;

   if (protbind_exp > 0) {
      R_plas = (pow(10.0, acidity * (pKa - pH_plas)));
      R_mucus = (pow(10.0, acidity * (pKa - nasal_pH)));
      m_higgins = wt_muc * (watmuc_fr * (1.0 + R_mucus) + protmuc_fr * (protbnd_cnstuf + (protbnd_cnstif * R_mucus)));
      for (i = 0; i <= rdnmber_compt - 1; i++) {
         R_ecf = (pow(10.0, acidity * (pKa - pH_ecf[i])));
         x_higgins[i] = (pow(10.0, acidity * (pKa - pH_cel[i])));
         b_higgins[i] = (celbld_fr * (erythplas_ratio - 1.0) + 1.0)
              * (protplas_fr * (protbnd_cnstuf + protbnd_cnstif * R_plas)
              + watplas_fr * (1.0 + R_plas));
         c_higgins[i] = celtis_fr[i] * ((lipfr_tiscelfr[i] * lipwat_part)
              + (protfr_tiscelfr[i] * (protbnd_cnstuf + protbnd_cnstif * x_higgins[i]))
              + watfr_tiscelfr[i] * (x_higgins[i] + 1.0));
         e_higgins[i] = (1.0 - celtis_fr[i]) * (protecf_fr[i] * (protbnd_cnstuf + (protbnd_cnstif * R_ecf))
              + watecf_fr[i] * (1.0 + R_ecf));
      }
   }
   else {
      R_plas = R_ecf = R_mucus = 1.0;
      for (i = 0; i <= rdnmber_compt - 1; i++)
         x_higgins[i] = b_higgins[i] = c_higgins[i] = e_higgins[i] = 1.0;
   }
}

void equilibrate(int rdnmber_compt, double *aa_mucus, double *aabt_mucus)
{
   int i;
   for (i = 0; i <= rdnmber_compt - 1; i++) {
      stanconc_old[i] = conc_higgins[i];
      tisdrg_begequ[i] = tisdrg_equ[i];
   }
   for (i = 0; i <= rdnmber_compt - 1; i++) {
      switch (i) {
         case PERIPHERAL_SHUNT:
              blddrg_begequ[i] = tis2comp_drgfr[i];
              tisdrg_equ[i] = 0.0;
              bldtisdrg_equ[i] = tis2comp_drgfr[i];
              conc_higgins[i] = tis2comp_drgfr[i] / (compsvmass_fr[i] * b_higgins[i]);
              break;
         case NASAL_MUCOSAL_SITE:
              *aabt_mucus = tis2comp_drgfr[i] + tisdrg_equ[i] + *aa_mucus;
              conc_higgins[i] = *aabt_mucus / ((compsvmass_fr[i] * b_higgins[i])
                  + tiscompmass_fr[i] * (e_higgins[i] + c_higgins[i])+ m_higgins);
              blddrg_begequ[i] = conc_higgins[i] * compsvmass_fr[i] * b_higgins[i];
              tisdrg_equ[i] = conc_higgins[i] * tiscompmass_fr[i] * (e_higgins[i] + c_higgins[i]);
              *aa_mucus = conc_higgins[i] * m_higgins;
              bldtisdrg_equ[i] = blddrg_begequ[i] + tisdrg_equ[i];
              break;
         case KIDNEYS:
              bldtisdrg_equ[i] = tis2comp_drgfr[i] + tisdrg_equ[i];
              b_higgins[i] /= urine_concfct;
              c_higgins[i] /= urine_concfct;
              e_higgins[i] /= urine_concfct;
              conc_higgins[i] = bldtisdrg_equ[i] / ((compsvmass_fr[i] * b_higgins[i])
                   + (tiscompmass_fr[i] * (c_higgins[i] + e_higgins[i])) + u_higgins);
              blddrg_begequ[i] = conc_higgins[i] * compsvmass_fr[i] * b_higgins[i];
              tisdrg_equ[i] = conc_higgins[i] * tiscompmass_fr[i] * (c_higgins[i] + e_higgins[i]);
              conc_higgins[i] /= urine_concfct;
              b_higgins[i] *= urine_concfct;
              c_higgins[i] *= urine_concfct;
              e_higgins[i] *= urine_concfct;
              break;
         default:
              bldtisdrg_equ[i] = tis2comp_drgfr[i] + tisdrg_equ[i];
              conc_higgins[i] = bldtisdrg_equ[i] / ((compsvmass_fr[i] * b_higgins[i])
                   + (tiscompmass_fr[i] * (c_higgins[i] + e_higgins[i])));
              blddrg_begequ[i] = conc_higgins[i] * compsvmass_fr[i] * b_higgins[i];
              tisdrg_equ[i] = conc_higgins[i] * tiscompmass_fr[i] * (c_higgins[i] + e_higgins[i]);
              break;
      }
   }
}

void model(int rdnmber_compt, int nmber_compt, double *total_dose,
         double *aa_mucus, double *aabt_mucus)
{
   int i;
   double temp = 0.0;
   //
   // Calculate fractions of stroke volume mass to all compartments.
   //
   for (i = 0; i <= rdnmber_compt - 1; i++)
        compsvmass_fr[i] = cardop_fr[i] * stroke_volweight;
   //
   // Calculate masses of fractions of stroke-volume (svw) to pools.
   //
   vsvfr2pool_mass = stroke_volweight;
   msvfr2pool_mass = compsvmass_fr[MUSCLE];
   fsvfr2pool_mass = compsvmass_fr[FAT];
   psvfr2pool_mass = compsvmass_fr[GUT_SPLEEN];
   //
   // Calculate the amounts of drug in svw fractions to compartments (in micro-moles).
   //
   for (i = 0; i <= rdnmber_compt - 1; i++) {
      if (i == LUNGS) tis2comp_drgfr[LUNGS] = (compsvmass_fr[LUNGS] / injpool_mass) * injpool_drg;
      else tis2comp_drgfr[i] = (compsvmass_fr[i] / artpool_mass) * artpool_drg;
   }
   //
   // Calculate the amounts of drug in svw fractions to pools (in micromoles).
   //
   ven2pool_drgfr = (vsvfr2pool_mass / venpool_mass) * venpool_drg;
   port2pool_drgfr = (psvfr2pool_mass / portpool_mass) * portpool_drg;
   muscl2pool_drgfr = (msvfr2pool_mass / musclpool_mass) * musclpool_drg;
   fat2pool_drgfr = (fsvfr2pool_mass / fatpool_mass) * fatpool_drg;
   //
   // Calculate the amounts of drug remaining in the pools (in micromoles).
   //
   portpool_drg -= port2pool_drgfr;
   musclpool_drg -= muscl2pool_drgfr;
   fatpool_drg -= fat2pool_drgfr;
   venpool_drg -= ven2pool_drgfr;
   injpool_drg -= tis2comp_drgfr[LUNGS];
   //
   // Calculate the amounts of drug remaining in the arterial pool (in micromoles).
   //
   for (i = 1; i <= rdnmber_compt - 1; i++) artpool_drg -= tis2comp_drgfr[i];
   //
   // Combine the portal pool with the liver in terms of fractions of stroke-volume 
   // mass and the amount of drug in the liver (in micromoles).
   //
   compsvmass_fr[LIVER] += psvfr2pool_mass;
   tis2comp_drgfr[LIVER] += port2pool_drgfr;
   //
   // Store the amount of drug in the "liver" in a temporary storage.
   //
   temp = tis2comp_drgfr[LIVER];
   //
   // remove the excreted drug from the blood entering the liver.
   //
   tis2comp_drgfr[LIVER] = tis2comp_drgfr[LIVER] * (1.0 - livmetex_fr);
   //
   // Keep record of the cumulative dose metabolised in the liver.
   //
   cumul_met += (temp - tis2comp_drgfr[LIVER]);
   //
   // Equilibrate tissue-drug with blood-drug in all tissue compartments.
   //
   equilibrate(rdnmber_compt, aa_mucus, aabt_mucus);
   //
   // Keep record of the cumulative dose excreted from the kidneys.
   //
   cumul_excr += (conc_higgins[KIDNEYS] * u_higgins * urine_concfct);
   //
   // Once equilibration is achieved update the drug amounts in pools by addition of
   // incoming fractions; sample brain and intra-muscular injection site included if need be.
   //
   artpool_drg += blddrg_begequ[LUNGS];
   portpool_drg += blddrg_begequ[GUT_SPLEEN];
   musclpool_drg += blddrg_begequ[MUSCLE];
   fatpool_drg += blddrg_begequ[FAT];
   injpool_drg += ven2pool_drgfr;
   venpool_drg += muscl2pool_drgfr + fat2pool_drgfr
         + blddrg_begequ[PERIPHERAL_SHUNT] + blddrg_begequ[KIDNEYS]
         + blddrg_begequ[LIVER] + blddrg_begequ[OTHER_VISCERA];
   if (rdnmber_compt >= (nmber_compt - 1)) {
      for (i = nmber_compt - 3; i <= rdnmber_compt - 1; i++)
         venpool_drg += blddrg_begequ[i];
   }
   //
   // Administers the intra-muscular dose as one increment to the intra-muscular injection site.
   //
   if (im_dose > 0.0) {
      tisdrg_equ[IM_INJECTION_SITE] += im_dose;
      *total_dose += im_dose;
      im_dose = 0.0;
   }
   //
   // Administers the intra-pulmonary dose as one increment to the lungs compartment.
   //
   if (ip_dose > 0.0) {
      tisdrg_equ[LUNGS] += ip_dose;
      *total_dose += ip_dose;
      ip_dose = 0.0;
   }
   //
   // Administers the iv dose increment to the injection pool.
   //
   injpool_drg += dose_increment;
   //
   // Update the total dose administered.
   //
   *total_dose += dose_increment;
}

void pend(double artpool_mass, double venpool_mass,
	  double injpool_mass, int rdnmber_compt, int protbind_exp)
{
   int i;
   double temp;
   //
   // Convert micrograms of iv-injected pethidine HCL to micromoles.
   //
   if (pend_ivdose > 0.0) {
      iv_dose = (pend_ivdose * 1.0) / (336.60);
      pend_ivdose = 0.0;
   }
   //
   // Convert micrograms of intra-muscular injected fentanyl to micromoles.
   //
   if (pend_imdose > 0.0 && pend_ivdose <= 0.0) {
      im_dose = pend_imdose / 336.60;
      pend_imdose = 0.0;
   }
   //
   // Convert micrograms of intra-pulmonary injected fentanyl to micromoles.
   //
   if (pend_ipdose > 0.0 && pend_ivdose <= 0.0 && pend_imdose <= 0.0) {
      ip_dose = pend_ipdose / 336.60;
      pend_ipdose = 0.0;
   }
   if (pend_infdur > 0.0) inf_duration = pend_infdur;
   if (pend_pHplas > 0.0) {
      pH_plas = pend_pHplas;
      for (i = 0; i <= rdnmber_compt - 1; i++) {
         if (i != 1) {
            pH_ecf[i] = pH_plas;
            pH_cel[i] = pH_ecf[i] + negoffset_pHt[i] + (negslope_pHt[i] * (pH_ecf[i] - 7.4));
         }
      }
      pH_ecf[PERIPHERAL_SHUNT] = 0.0;
      pH_cel[PERIPHERAL_SHUNT] = 0.0;
      lamset(rdnmber_compt, protbind_exp);
   }
   //
   // Check for pending cycle-time(s) and cardiac output(s).
   //
   if (pend_cycle > 0.0) cycle_time = pend_cycle;
   if (pend_cardop > 0.0) stroke_volweight = pend_cardop * (1.06/ 60.0) * cycle_time;
   //
   // If either cycle-time or cardiac output change test the svw against the smallest 
   // of the three blood pools in order to avoid pulling a volume fraction of blood 
   // which is larger than is physically possible.
   //
   if ((pend_cycle > 0.0 || pend_cardop > 0.0)
         && (((min(min(injpool_mass, artpool_mass), min(venpool_mass, stroke_volweight)))) != stroke_volweight)) {
      cycle_time = (min(min(injpool_mass, artpool_mass), min(venpool_mass, stroke_volweight)))
            * (1.0 /(pend_cardop * (1.06/ 60.0)));
      i = cycle_time;
      if ((cycle_time - i) >= 0.50) cycle_time = i + 1.0;
      else  cycle_time = i;
   }
   temp = (cycle_time * urine_flow) / (1000.0 * 60.0 * 60.0);
   u_higgins = temp * (1.0 + pow(10.0, acidity * (pKa - urine_pH)));
}

void dacon(double *protmuc_fr, double *protplas_fr)
{
   int i;
   //
   // Calculate the mass of blood from the total body mass.
   //
   blood_weight = body_weight * bldtotmass_fr;
   //
   // Calculate tissue compartment masses.
   //
   for (i = 0; i <= rdnmber_compt - 1; i++) tiscompmass_fr[i] = body_weight * tistotmass_fr[i];
   //
   // Calculate the blood pool masses.
   //
   artpool_mass = artpool_bfr * blood_weight;
   portpool_mass = portpool_bfr * blood_weight;
   musclpool_mass = musclpool_bfr * blood_weight;
   fatpool_mass = fatpool_bfr * blood_weight;
   venpool_mass = venpool_bfr * blood_weight;
   injpool_mass = injpool_bfr * blood_weight;
   //
   // Calculate the water, fat and protein fractions in the tissue cell fraction, then convert
   // the protein-cell and protein-ecf fractions to micro-moles of protein for each compartment.
   //
   for (i = 0; i <= rdnmber_compt - 1; i++) {
      if (i != PERIPHERAL_SHUNT) {
         watfr_tiscelfr[i] = (wattis_fr[i] - (watecf_fr[i] * (1.0 - celtis_fr[i]))) / (celtis_fr[i]);
         lipfr_tiscelfr[i] = (liptis_fr[i] - (lipecf_fr[i] * (1.0 - celtis_fr[i]))) / (celtis_fr[i]);
         protfr_tiscelfr[i] = (prottis_fr[i] - (protecf_fr[i] * (1.0 - celtis_fr[i]))) / (celtis_fr[i]);
         protecf_fr[i] = (protecf_fr[i] / ecfprot_mlms) * (pow(10.0, 9.0));
         protfr_tiscelfr[i] = (protfr_tiscelfr[i] / celprot_mlms[i]) * (pow(10.0, 9.0));
      }
   }
   watfr_tiscelfr[PERIPHERAL_SHUNT] = 0.0;
   lipfr_tiscelfr[PERIPHERAL_SHUNT] = 0.0;
   protfr_tiscelfr[PERIPHERAL_SHUNT] = 0.0;
   protecf_fr[PERIPHERAL_SHUNT] = 0.0;
   *protmuc_fr = (*protmuc_fr / mucprot_mlms) * (pow(10, 9));
   *protplas_fr = (*protplas_fr / plasprot_mlms) * (pow(10, 9));
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
   #define Data_IN	  prhs[0]
   #define NumCoef_IN	  prhs[1]
   #define DenCoef_IN	  prhs[2]
   #define Effect_IN	  prhs[3]
   #define TFilt_IN	  prhs[4]
   #define Time_OUT       plhs[0]
   #define IODat_OUT      plhs[1]
   #define IDPars_OUT     plhs[2]
   #define ObjFun_OUT     plhs[3]
   #define Critr_OUT      plhs[4]

   double *y_Time, *y_IODat, *y_IDPars, *y_ObjFun, *y_Critr;
   double *Dat_input, *Dat_NumCoef, *Dat_DenCoef, *Dat_Effect, *Dat_TFilter;

   double Offset = 0.0, dtemp;
   int count, isampl=10, cycle_number, k_noise;
   double proof = 1.0e+30, v_noise=0;
   int ix, jx, m;

   double ax, bx, cx, fa, fb, fc, xmin, gold, amin[300];
   int iflag, nmin = 0;

   Dat_input = mxGetPr(Data_IN);
   m = Dat_input[0];
   tun = Dat_input[1];
   DELAY = Dat_input[2];
   idelm=DELAY;
   High = Dat_input[3];
   na = Dat_input[4];
   nb = Dat_input[5];
   gamma_noise = Dat_input[6];
   isampl = Dat_input[7];
   ah = Dat_input[8];
   k_noise=0;
   if (mxGetN(Data_IN)==10) k_noise=1;

   pend_infdur = isampl*ah;

   n_num=mxGetN(NumCoef_IN)-1;
   n_den=mxGetN(DenCoef_IN)-1;
   Dat_NumCoef = mxGetPr(NumCoef_IN);
   Dat_DenCoef = mxGetPr(DenCoef_IN);
   keyDyna = 1;
   if (n_den==-1) {
      keyDyna=0;
      read_parameters();
      body_weight = Dat_NumCoef[0];
      pend_cardop = Dat_NumCoef[1];
      dacon(&protmuc_fr, &protplas_fr);
      pend(artpool_mass, venpool_mass, injpool_mass,rdnmber_compt, protbind_exp);
   }
   else {

// Monic processing of Transfer functions
      for (i=0; i<=n_num; i++) num[i]=Dat_NumCoef[i]/Dat_DenCoef[0];
      for (i=n_den; i>=0; i--) den[i]=Dat_DenCoef[i]/Dat_DenCoef[0];
      ND=n_den;
   }
   Dat_Effect = mxGetPr(Effect_IN);
   sys_x50=Dat_Effect[0];
   mdl_x50=Dat_Effect[1];
   sys_pow=Dat_Effect[2];
   mdl_pow=Dat_Effect[3];

   if (nrhs==5){
      Dat_TFilter = mxGetPr(TFilt_IN);
      for (i=0; i<=2; i++) t[i]=Dat_TFilter[i];
      t0=t[0]; t1=t[1]; t2=t[2];
   }
   else {
      t0=1; t1=-1.6; t2=0.64; 
      t[0]=t0; t[1]=t1; t[2]=t2;
   }

   Time_OUT       = mxCreateDoubleMatrix(m,1, mxREAL);
   IODat_OUT      = mxCreateDoubleMatrix(m,5, mxREAL);
   IDPars_OUT     = mxCreateDoubleMatrix(m,na+nb, mxREAL);
   ObjFun_OUT     = mxCreateDoubleMatrix(m,5, mxREAL);
   Critr_OUT      = mxCreateDoubleMatrix(3,1, mxREAL);

   y_Time = mxGetPr(Time_OUT);
   y_IODat = mxGetPr(IODat_OUT);
   y_IDPars = mxGetPr(IDPars_OUT);
   y_ObjFun = mxGetPr(ObjFun_OUT);
   y_Critr = mxGetPr(Critr_OUT);

   napp = na + 2;
   nao = na +npd + 2;
   npdp = npd + 1;
   npt = npd + nt + 1;
   nptm=nao; if (nao<npt) nptm=npt;
   it = idelm + 1;
   itb = idelm + iov + nb;

   for (i = 0; i <= na + nb + iov - 1; i++) diag[i] = alc;
   pn[0] = 1; for (i=1; i<=na; i++) pn[i] = 0;
   pd[0] = 1; for (i=1; i<=nb; i++) pd[i] = 0;

//  higgins model initialization

   a[0] = 1;
   for (i=0; i<=n2; i++) omega_0[i]=0.8;

   for (i = 0; i <= 9; i++) prm[i] = 0;
   prm[nb] = 1;

   for (cycle_number=0; cycle_number<m; cycle_number++){
      if (keyDyna==1) {
         for (count=0; count<isampl; count++) rk4(ry, ah, ryout, derivs);
         output_0=0;
         for (i=n_num; i>=0; i--) output_0+=num[i]*ry[n_num-i];
      }
      else {
         for (run_time = 0.0; run_time < pend_infdur; run_time += cycle_time){
            model(rdnmber_compt, nmber_compt, &total_dose, &aa_mucus, &aabt_mucus);
         }
         output_0 = 336.6*tisdrg_equ[BRAIN]/tiscompmass_fr[BRAIN];
      }
      output = pow(output_0, sys_pow) / (pow(output_0, sys_pow) + pow(sys_x50, sys_pow));
      if (gamma_noise>0) {
         if (k_noise==0) v_noise=(1 - (2 * ran2(&idum))) * gamma_noise;
         else v_noise = gasdev(&idum) * gamma_noise;
         output += v_noise;
      }
      ise += pow((omega - output), 2);
      if (output <= 0) output = 0;
      if (output >= 1) output = 1;

      if (output <= 0) y[19] = 0;
      if (output>0 && output<1) y[19]=mdl_x50*(pow(output/(1-output), 1/mdl_pow));
      yf[19] = y[19] - y[18];
      yd[19] = (y[19] / pd[0]) - (pd[1] / pd[0]) * yd[18] - (pd[2] / pd[0]) * yd[17];
      yfdt[19] = yd[19]/t0 - t1/t0*yfdt[18] - t2/t0*yfdt[17] - t3/t0*yfdt[16];
      dyft[19] = yf[19]/t0 - t1/t0*dyft[18] - t2/t0*dyft[17] - t3/t0*dyft[16];
      dyft_rls[19] = yf[19]/tf0 - tf1/tf0*dyft_rls[18] - tf2/tf0*dyft_rls[17] - tf3/tf0*dyft_rls[16];

      mea_vec(dyft_rls, duft_rls);
      pe = dyft_rls[19];
      for (i = 0; i <= na + nb + iov - 1; i++) pe -= prm[i] * phi[i];
      dtemp = 1;
      for (i = 0; i <= nb-1; i++) dtemp += prm[i];
      gain = 0;
      for (i = nb; i <= na+nb-1; i++) gain += prm[i];
      gain = gain / dtemp;

      udf(pe, phi, diag, kalgain, rho, cov, (na + nb + iov), prm);

/*      if (Time >= 100 && Time <= 200) omega = 0.95;
      else if (Time > 200 && Time <= 250) omega = 0.90;
      else if(Time > 250) omega = 0.95;
*/
      if (Time < 100-n2) omega_0[n2-1] = 0.8;
      else if (Time >= 100-n2 && Time <= 200-n2) omega_0[n2-1] = 0.95;
      else if (Time > 200-n2 && Time <= 250-n2) omega_0[n2-1] = 0.90;
      else if(Time > 250-n2) omega_0[n2-1] = 0.95;

      str_prm();  solv_dioph();  g_elements();  vv_calc();

      proof = 1.0e+30;

      for (ix = - 50; ix <= 50; ix++) {
         if (ix != 0) {
            ax = ix;
            bx = ix + 1;
            mnbrak(&ax, &bx, &cx, &fa, &fb, &fc, obj_func);
            gold = golden(ax, bx, cx, obj_func, TOL, &xmin);
            if (nmin == 0) {
               amin[1] = xmin;
               nmin = 1;
            }
            else {
               iflag = 0;
               for (jx=1; jx<=nmin; jx++) if (fabs(xmin-amin[jx])<=EQL*xmin) iflag=1;
               if (iflag == 0) amin[++nmin] = xmin;
            }
        }
        if (obj_func(xmin) < proof) {
           proof = obj_func(xmin);
           delta = xmin;
        }
     }

     nmin = 0;
     Global_Minimum = delta;

     u[19] = u[18] + delta;
     if (High>0) if (u[19] >= High) u[19] = High; 
     if (u[19] <= Low) u[19] = Low; 
     delta = u[19] - u[18];
     uf[19] = delta;
     duft[19] = uf[19]/t0 - t1/t0*duft[18] - t2/t0*duft[17] - t3/t0*duft[16];
     duft_rls[19] = uf[19]/tf0 - tf1/tf0*duft_rls[18] - tf2/tf0*duft_rls[17] - tf3/tf0*duft_rls[16];
     u_out = u[19 - DELAY];

// combine Higgins model
      dose_increment = u_out/(cycle_time*336.6);

      y_Time[cycle_number]=Time;
      y_IODat[cycle_number]=omega_0[0];
      y_IODat[cycle_number+m]=u[19];
      y_IODat[cycle_number+2*m]=output;
      y_IODat[cycle_number+3*m]=gain;
      y_IODat[cycle_number+4*m]=output_0;
      for (i=0; i<=na+nb-1; i++) y_IDPars[cycle_number+i*m]=prm[i];
      y_ObjFun[cycle_number]=Global_Minimum;
      y_ObjFun[cycle_number+m]=obj_func(Global_Minimum);
      y_ObjFun[cycle_number+2*m]=delta;
      y_ObjFun[cycle_number+3*m]=pend_ivdose;
      y_ObjFun[cycle_number+3*m]=obj_func(delta);

      shift_data();
      Time += (isampl * ah);
      sum_u += u_out*isampl*ah;
      itae += Time * fabs(omega - output);
   }
   y_Critr[0]=ise;
   y_Critr[1]=itae;
   y_Critr[2]=sum_u*60;
}
