clear higginsm;

key=figflag('Fentanyl Modelling & Analysis Program');

% Parameters initialization

bd_wt=70; % body weight of the subject, set the nominal
card_out=6.48; % cardiac output of the subject, set the nominal
t_f=21600; % end of simulation (seconds), set the nominal
FenDose=100; %Total does in mg's
DoseDur=60; % Total injection duration in seconds
DT=1; % calculation step in the algebraic model in seconds
keyModType=2;

% control parameters
nn_lambda=0; nn_umax=0.2; nn_simpoints=300;
nn_gamma_n=0; nn_ie50=6.9; nn_gamma=4.3;
nn_TFilter=[1,-1.6,0.64];
keyCtrl=0;

%calculate SIMULINK input variables
T_t=[0; DoseDur; DoseDur+0.001; t_f]/60; 
U=[1;1;0;0]*FenDose/DoseDur;

bAxisLog=1; % axis format, where
   % bAxisLog=1 for loglog, bAxisLog=0 for linear and 2 for Log-X
bDispCur=3; % show signals, where
   % bDispCur=1 for albegraic, 2 for SIMULINK and 3 for both   
bDispConc=2; % display what, where
   % bDispConc=1 for drug amount, and 2 for blood concentration and
   % 3 for plasma concentration
bAlgeModel=0; %existence of algebraic model
bFitModel=0; %type of fitting, where 0 for non-exist,
   %1 for direct identification, and 2 for interpolation.
bShwPlt=10; % show plot of
   %  1 for lungs
   %  2 for peripheral shunt
   %  3 for kidneys
   %  1 for 
   %  1 for lungs
   %  1 for lungs
   %  1 for lungs
   %  1 for lungs
   %  1 for lungs
   %  1 for lungs
   %  1 for lungs
   %  1 for lungs
   %  1 for lungs
bFigExist=0;
bUnits=1;
%bUnits =1 for micro mols, and 2 for ng
bEffect=1; % 1 for EEG reading \in (0,1), 2 for spectral edge
EC50=6.9;
gam=4.9;
EMax=15;
E0=20;
nn_order=4; %expected order of reduction

hMain=figure('MenuBar','none',...
      'Position',[150 170 400 280],...
      'NumberTitle','off',...
      'Name','Fentanyl Modelling & Analysis Program',...
      'Resize','off');
   
%design the model menu   
mnuModel(1)=uimenu('Label','Model');
mnuModel(2)=uimenu(mnuModel(1),'Label','Parameters',...
      'Callback','hig_vars');
mnuModel(3)=uimenu(mnuModel(1),...
      'Label','Run Algebraic model',...
      'Callback',['if bAlgeModel==0, ',...
                     'clear higginsm; ',...
                     '[nn,t_Alge,Lungs,Perip,Kidneys,',...
                        'GutSpleen,Liver,OtherVis,Muscle,Fat,',...
                        'Nasal,Brain,ImSite,Metabolism,Excretion,',...
                        'ArtPool,VenPool,BldFrac]=',...
                           'higginsm([bd_wt,t_f,DT,',...
                              'card_out,FenDose,DoseDur]);',...
                  'end, ',...
                  'bAlgeModel=1; ']);
mnuModel(4)=uimenu(mnuModel(1),...
      'Label','Identification',...
      'Callback',['bFitModel=1; ',...
                  'eval(get(mnuModel(3),''CallBack'')); ',...
                  'ident_tf']);
mnuModel(5)=uimenu(mnuModel(1),...
      'Label','Establish SIMULINK Model');
mnuModel(8)=uimenu(mnuModel(5),...
      'Label','Direct Identification',...
      'CallBack','eval(get(mnuModel(4),''CallBack''));');
mnuModel(9)=uimenu(mnuModel(5),...
      'Label','3D Linear Interpolation',...
      'CallBack',['x=bd_wt; y=card_out;',...
         'bFitModel=2; get_hig_tf_3; ']);
mnuModel(6)=uimenu(mnuModel(1),...
      'Label','Show Parameters');
mnuModShw(1)=uimenu(mnuModel(6),...
      'Label','Transfer Functions',...
      'Callback',['if bFitModel==0,',...
                     'eval(get(mnuModel(9),''CallBack''));',...
                  'end, ',...   
                  'disp(''Fat Model''); G_Fat, ',...
                  'disp(''Lungs Model''); G_Lungs, ',...
                  'disp(''Gut Spleen Model''), G_Gut,',...
                  'disp(''Kidneys Model''), G_Kidneys, ',...
                  'disp(''Liver Model''); G_Liver,',...
                  'disp(''Other Viscera Model''), G_OtherVis, ',...
                  'disp(''Muscle Model''), G_Muscle, ',...
                  'disp(''Brain Model''), G_Brain, ',...
                  'disp(''Nasal Model''), G_Nasal']);
mnuModShw(2)=uimenu(mnuModel(6),...
      'Label','Concentration Ratios',...
      'Callback',['if bFitModel==0,',...
                     'eval(get(mnuModel(9),''CallBack''));',...
                  'end, ',... 
                  'disp([''Kidneys ratio='',num2str(K_Opts(1))]),',...
                  'disp([''Liver ratio='',num2str(K_Opts(2))]),',...   
                  'disp([''Gut Spleen ratio='',num2str(K_Opts(3))]),',... 
                  'disp([''Other Viscera ratio='',num2str(K_Opts(4))]),',...
                  'disp([''Muscle ratio='',num2str(K_Opts(5))]),',...   
                  'disp([''Fat ratio='',num2str(K_Opts(6))]),',...   
                  'disp([''Brain ratio='',num2str(K_Opts(7))]),',...   
                  'disp([''Nasal ratio='',num2str(K_Opts(8))]),',...   
                  'disp([''Venous Pool ratio='',num2str(K_Opts(9))]),',...   
                  'disp([''Arterial Pool ratio='',num2str(K_Opts(10))]),',...   
                  'disp([''Lungs ratio='',num2str(K_Opts(11))])']);
            
mnuModel(7)=uimenu(mnuModel(1),...
      'Label','Exit',...
      'separator','on',...
      'Callback','close(gcf)');
   
%design the Display menu   
mnuDisplay(1)=uimenu('Label','Display');
mnuDisplay(2)=uimenu(mnuDisplay(1),...
      'Label','Lungs',...
      'Callback','bShwPlt=11; shw_plot');
mnuDisplay(3)=uimenu(mnuDisplay(1),...
      'Label','Kidneyss',...
      'Callback','bShwPlt=1; shw_plot');
mnuDisplay(4)=uimenu(mnuDisplay(1),...
      'Label','Liver',...
      'Callback','bShwPlt=2; shw_plot');
mnuDisplay(5)=uimenu(mnuDisplay(1),...
      'Label','Gut Spleen',...
      'Callback','bShwPlt=3; shw_plot');
mnuDisplay(6)=uimenu(mnuDisplay(1),...
      'Label','Other Viscera',...
      'Callback','bShwPlt=4; shw_plot');
mnuDisplay(7)=uimenu(mnuDisplay(1),...
      'Label','Muscle',...
      'Callback','bShwPlt=5; shw_plot');
mnuDisplay(8)=uimenu(mnuDisplay(1),...
      'Label','Fat',...
      'Callback','bShwPlt=6; shw_plot');
mnuDisplay(9)=uimenu(mnuDisplay(1),...
      'Label','Brain',...
      'Callback','bShwPlt=7; shw_plot');
mnuDisplay(10)=uimenu(mnuDisplay(1),...
      'Label','Nasal',...
      'Callback','bShwPlt=8; shw_plot');
mnuDisplay(11)=uimenu(mnuDisplay(1),...
      'Label','Venous Pool',...
      'Callback','bShwPlt=9; shw_plot');
mnuDisplay(12)=uimenu(mnuDisplay(1),...
      'Label','Arterial Pool',...
      'Callback','bShwPlt=10; shw_plot');
   
mnuOverall(1)=uimenu(mnuDisplay(1),...
      'Separator','on',...
      'Label','Overrall Model');
mnuOverall(2)=uimenu(mnuOverall(1),...
      'Label','Transfer Function',...      
      'Check','off',...
      'Callback',['keyModType=1; ',...
         'set(mnuOverall(2),''Check'',''on'');',...
         'set(mnuOverall([3,4]),''Check'',''off'');',...
         'get_overall_model; mod_display;']);
mnuOverall(3)=uimenu(mnuOverall(1),...
      'Label','Pole-zero-gain',...      
      'Check','on',...
      'Callback',['keyModType=2; ',...
         'set(mnuOverall(3),''Check'',''on'');',...
         'set(mnuOverall([2,4]),''Check'',''off'');',...
         'get_overall_model; mod_display;']);
mnuOverall(4)=uimenu(mnuOverall(1),...
      'Label','State Space',...      
      'Check','off',...
      'Callback',['keyModType=3; ',...
         'set(mnuOverall(4),''Check'',''on'');',...
         'set(mnuOverall([2,3]),''Check'',''off'');',...
         'get_overall_model; mod_display;']);
mnuOverall(5)=uimenu(mnuDisplay(1),...
      'Label','Pole-zero Cancellation');
mnuOverall(6)=uimenu(mnuOverall(5),...
      'Label','with default criterion',...      
      'Check','off',...   
      'Callback',['keyMinireal=1; ',...
                  'set(mnuOverall(6),''Check'',''on'');',...
                  'set(mnuOverall([7:10]),''Check'',''off'');',...
                  'pole_zero_cancel; mod_display;']);
mnuOverall(7)=uimenu(mnuOverall(5),...
      'Label','with 1e-5 criterion',...      
      'Check','off',...   
      'Callback',['keyMinireal=2; ',...
                  'set(mnuOverall(7),''Check'',''on'');',...
                  'set(mnuOverall([6,8:10]),''Check'',''off'');',...
                  'pole_zero_cancel; mod_display;']);
mnuOverall(8)=uimenu(mnuOverall(5),...
      'Label','with 1e-4 criterion',...      
      'Check','off',...   
      'Callback',['keyMinireal=3; ',...
                  'set(mnuOverall(8),''Check'',''on'');',...
                  'set(mnuOverall([6,7,9,10]),''Check'',''off'');',...
                  'pole_zero_cancel; mod_display;']);
mnuOverall(9)=uimenu(mnuOverall(5),...
      'Label','with 1e-3 criterion',...      
      'Check','off',...   
      'Callback',['keyMinireal=4; ',...
                  'set(mnuOverall(9),''Check'',''on'');',...
                  'set(mnuOverall([6:8,10]),''Check'',''off'');',...
                  'pole_zero_cancel; mod_display;']);
mnuOverall(10)=uimenu(mnuOverall(5),...
      'Label','with 1e-2 criterion',...      
      'Check','off',...   
      'Callback',['keyMinireal=5; ',...
                  'set(mnuOverall(10),''Check'',''on'');',...
                  'set(mnuOverall([6:9]),''Check'',''off'');',...
                  'pole_zero_cancel; mod_display;']);
            
mnuReduct(1)=uimenu(mnuDisplay(1),...
      'Label','Model Reduction',...
      'CallBack','get_reduction;');
   
mnuDispSub(1)=uimenu(mnuDisplay(1),...
      'Label','Display Format',...
      'separator','on');
mnuDispSub(2)=uimenu(mnuDispSub(1),...
      'Label','Drug Amount',...
      'Checked','off',...
      'Callback',['bDispConc=1; ',...
         'set(mnuDispSub([3,11]),''Checked'',''off'');',...
         'set(mnuDispSub(2),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
mnuDispSub(3)=uimenu(mnuDispSub(1),...
      'Label','Blood Concentration',...
      'Checked','on',...
      'Callback',['bDispConc=2; ',...
         'set(mnuDispSub([2,11]),''Checked'',''off'');',...
         'set(mnuDispSub(3),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
mnuDispSub(11)=uimenu(mnuDispSub(1),...
      'Label','Plasma Concentration',...
      'Checked','off',...
      'Callback',['bDispConc=3; ',...
         'set(mnuDispSub(2:3),''Checked'',''off'');',...
         'set(mnuDispSub(11),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
mnuDispSub(4)=uimenu(mnuDisplay(1),'Label','Axis Scale');
mnuDispSub(5)=uimenu(mnuDispSub(4),...
      'Label','Linear scale',...
      'Checked','off',...
      'Callback',['bAxisLog=0; ',...
         'set(mnuDispSub([6,11]),''Checked'',''off'');',...
         'set(mnuDispSub(5),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
mnuDispSub(6)=uimenu(mnuDispSub(4),...
      'Label','LogLog scale',...
      'Checked','on',...
      'Callback',['bAxisLog=1; ',...
         'set(mnuDispSub([5,11]),''Checked'',''off'');',...
         'set(mnuDispSub(6),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
mnuDispSub(11)=uimenu(mnuDispSub(4),...
      'Label','Logarithmic &X-Axis',...
      'Checked','off',...
      'Callback',['bAxisLog=2; ',...
         'set(mnuDispSub(5:6),''Checked'',''off'');',...
         'set(mnuDispSub(11),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
mnuDispSub(7)=uimenu(mnuDisplay(1),'Label','Display Curves of');
mnuDispSub(8)=uimenu(mnuDispSub(7),...
      'Label','&Algebraic model',...
      'Checked','off',...
      'Callback',['bDispCur=1; ',...
         'set(mnuDispSub(9:10),''Checked'',''off'');',...
         'set(mnuDispSub(8),''Checked'',''on'');',...
         'bFigExist, if bFigExist==1, shw_plot; end']);
mnuDispSub(9)=uimenu(mnuDispSub(7),...
      'Label','&SIMULINK model',...
      'Checked','off',...
      'Callback',['bDispCur=2; ',...
         'set(mnuDispSub([8,10]),''Checked'',''off'');',...
         'set(mnuDispSub(9),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
mnuDispSub(10)=uimenu(mnuDispSub(7),...
      'Label','&Both',...
      'Checked','on',...
      'Callback',['bDispCur=3; ',...
         'set(mnuDispSub(8:9),''Checked'',''off'');',...
         'set(mnuDispSub(10),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
mnuDispSub(12)=uimenu(mnuDisplay(1),'Label','&Units');
mnuDispSub(13)=uimenu(mnuDispSub(12),...
      'Label','&Micro Mols',...
      'Checked','off',...
      'Callback',['bUnits=1; ',...
         'set(mnuDispSub(14),''Checked'',''off'');',...
         'set(mnuDispSub(13),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
mnuDispSub(14)=uimenu(mnuDispSub(12),...
      'Label','&Nano Grams',...
      'Checked','off',...
      'Callback',['bUnits=2; ',...
         'set(mnuDispSub(13),''Checked'',''off'');',...
         'set(mnuDispSub(14),''Checked'',''on'');',...
         'if bFigExist==1, shw_plot; end']);
            
mnuEffect(1)=uimenu('Label','Effect');
mnuEffect(2)=uimenu(mnuEffect(1),'Label','EEG Direct',...
         'CallBack','bEffect=1; shw_effect;');
mnuEffect(3)=uimenu(mnuEffect(1),'Label','Spectral Edge',...
         'CallBack','bEffect=2; shw_effect;');
mnuEffect(4)=uimenu(mnuEffect(1),'Label','NLGPC Control',...
         'CallBack','nlgpc_ctrl;');

mnuFigure(1)=uimenu('Label','Figure');
mnuFigSub(1)=uimenu(mnuFigure(1),'Label','Copy Figure');
mnuFigSub(2)=uimenu(mnuFigSub(1),...
   'Label','Copy As &Meta Format',...
   'CallBack','print -dmeta');
mnuFigSub(3)=uimenu(mnuFigSub(1),...
   'Label','Copy As &BitMap Format',...
   'CallBack','print -dbitmap');
mnuFigSub(12)=uimenu(mnuFigSub(1),...
   'Label','Save As &EPS File',...
   'CallBack',...
     ['[PrtFile,PrtPath]=uiputfile(''*.eps'');',...
      'if PrtFile~=0, ',...
         'eval([''print -deps2 -adobecset '' PrtPath,PrtFile]); ',...
      'end']);

mnuFigSub(4)=uimenu(mnuFigure(1),'Label','&Axis and Grid');
mnuFigSub(5)=uimenu(mnuFigSub(4),...
   'Label','&Grid on',...
   'CallBack',['xx=get(gca,''xgrid'');',...
         'if strcmp(xx,''on''), xx0=''off'';',...   
         'else, xx0=''on''; end,',...
         'set(mnuFigSub(5),''Label'',[''&Grid '' xx]);',...
         'set(gca,''xgrid'',xx0); set(gca,''ygrid'',xx0);']);
mnuFigSub(6)=uimenu(mnuFigSub(4),...
   'Label','&Box off',...
   'CallBack',['xx=get(gca,''box'');',...
         'if strcmp(xx,''on''), xx0=''off'';',...   
         'else, xx0=''on''; end,',...
         'set(mnuFigSub(6),''Label'',[''&Box '' xx]);',...
         'set(gca,''box'',xx0);']);
mnuFigSub(7)=uimenu(mnuFigure(1),'Label','&Zooming');
mnuFigSub(8)=uimenu(mnuFigSub(7),...
   'Label','with &Zooming',...
   'CallBack','zoom on');
	mnuFigSub(12)=uimenu(mnuFigSub(7),...
   'Label','&No Zooming',...
   'CallBack','zoom off');
mnuFigSub(9)=uimenu(mnuFigSub(7),...
   'Label','Zooming &X-axis',...
   'CallBack','zoom xon');
mnuFigSub(10)=uimenu(mnuFigSub(7),...
   'Label','Zooming &Y-axis',...
   'CallBack','zoom yon');

hMSubOption(1)=uimenu(mnuFigure(1),...
      'Label','&Cursor Positioning',...
      'CallBack',['[x0,y0,but0]=getlegnd(1); ',...
         'str=['' ('' num2str(x0), '','', num2str(y0) '')'']; ',...
         'bHeld=0; if ishold, bHeld=1; end, ',...
         'hold on; plot(x0,y0,''o''); ',...
         'text(x0,y0,str); ',...
         'if ~bHeld, hold off; end']);
hMLegends(1)=uimenu(mnuFigure(1),...
      'Label','&Legends');
hMLegends(2)=uimenu(hMLegends(1),...
      'Label','&New Legend',...
      'Callback','nTask=1; hg_legends_proc;');
hMLegends(3)=uimenu(hMLegends(1),...
      'Label','&Edit Legend',...
      'Callback','nTask=2; hg_legends_proc;');
hMLegends(4)=uimenu(hMLegends(1),...
      'Label','&Move Legend',...
      'Callback','nTask=3; hg_legends_proc;');
hMLegends(5)=uimenu(hMLegends(1),...
      'Label','&Delete Legend',...
      'Callback','nTask=4; hg_legends_proc;');
hMLegends(6)=uimenu(hMLegends(1),...
      'Label','&Add a Line',...
      'Callback','nTask=5; hg_legends_proc;');
hMLegends(8)=uimenu(hMLegends(1),...
      'Label','Add an Arro&w',...
      'Callback','nTask=8; hg_legends_proc;');

mnuHelp(1)=uimenu('Label','&Help');
mnuHelp(2)=uimenu(mnuHelp(1),...
   'Label','About HigModel ...',...
   'CallBack','hm_about');

hLegd(1)=uicontrol('Style','Text',...
      'Units','normalize',...
      'Position',[0.02,0.9,0.15,0.06],...
      'String','Legends:',...
      'HorizontalAlignment','left',...
      'BackgroundColor','c',...
      'Visible','off');
hLegd(2)=uicontrol('Style','Edit',...
      'Units','normalize',...
      'Position',[0.20,0.9,0.6,0.06],...
      'HorizontalAlignment','left',...
      'String','',...
      'HorizontalAlignment','left',...
      'BackgroundColor',[1,1,1],...
      'Visible','off');
hLegd(3)=uicontrol('Style','Pushbutton',...
      'Units','normalize',...
      'Position',[0.83,0.88,0.13,0.08],...
      'String','Accept',...
      'Visible','off',...
      'CallBack',['hLegd=get(gcf,''UserData'');',...
        'str=get(hLegd(2),''String'');',...
        'set(hLegd(1:3),''Visible'',''off'');',...
        'if length(str)>0 & nTask==1,',...
           'nCol=get(gca,''XColor''); ',...
           '[x,y]=ginput(1);',...
           '[x,hh]=str2disp(x,y,str,nCol);',...
           'set(hh,''UserData'',[1,nCol]);',...
        'elseif nTask==2,',...
           'nCol=get(iLegends,''UserData'');',...
           'set(iLegends,''String'',str);',...
           'set(iLegends,''Color'',nCol(2:4));',...
        'end;']);

set(gcf,'UserData',hLegd);
