function c6exsf1_l2(block)
% Level-2 M file S-Function for limited integrator demo.
%   Copyright 1990-2004 The MathWorks, Inc.
%   $Revision: 1.1.6.1 $ 

  setup(block);
  
%endfunction

function setup(block)
  
  %% Register number of dialog parameters   
  block.NumDialogPrms = 4;
  
  %% Register number of input and output ports
  block.NumInputPorts  = 1; block.NumOutputPorts = 2;
  B=block.DialogPrm(2).Data; C=block.DialogPrm(3).Data;

  %% Setup functional port properties to dynamically
  %% inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
  block.InputPort(1).Dimensions = size(B,2);
  block.InputPort(1).DirectFeedthrough = true;
  
  block.OutputPort(1).Dimensions       = size(C,1);
  block.OutputPort(2).Dimensions       = size(B,1);

  %% Set block sample time to continuous
  block.SampleTimes = [0 0];
  block.NumContStates = size(B,1);

  %% Register methods
  block.RegBlockMethod('SetInputPortSamplingMode',@SetInputPortSamplingMode);
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('Derivatives',             @Derivatives);  
%endfunction

function SetInputPortSamplingMode(block, port, sp)
block.InputPort(port).SamplingMode = sp;
block.OutputPort(1).SamplingMode = sp;
block.OutputPort(2).SamplingMode = sp;

function Output(block)
C=block.DialogPrm(3).Data; D=block.DialogPrm(4).Data;
block.OutputPort(1).Data = C*block.ContStates.Data+D*block.InputPort(1).Data;
block.OutputPort(2).Data = block.ContStates.Data;
  
function InitConditions(block)
A=block.DialogPrm(1).Data; block.ContStates.Data = zeros(length(A),1);

function Derivatives(block)
A=block.DialogPrm(1).Data; B=block.DialogPrm(2).Data;
block.Derivatives.Data=A*block.ContStates.Data+B*block.InputPort(1).Data;
