function han_td_l2(block)
% Level-2 M file S-Function for limited integrator demo.
%   Copyright 1990-2004 The MathWorks, Inc.
%   $Revision: 1.1.6.1 $ 

  setup(block);
  
%endfunction

function setup(block)
  
  %% Register number of dialog parameters   
  block.NumDialogPrms = 3;

  %% Register number of input and output ports
  block.NumInputPorts  = 1;
  block.NumOutputPorts = 1;
  block.NumContStates = 0;

  %% Setup functional port properties to dynamically
  %% inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
 
  block.InputPort(1).Dimensions        = 1;
  block.InputPort(1).DirectFeedthrough = false;
  
  block.OutputPort(1).Dimensions       = 2;
  T = block.DialogPrm(3).Data;

  %% Set block sample time to continuous
  block.SampleTimes = [T 0];
  %% Register methods
  block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('Update',                  @Update);  
  
function DoPostPropSetup(block)
  %% Setup Dwork
  block.NumDworks = 1;
  block.Dwork(1).Name = 'x0'; 
  block.Dwork(1).Dimensions      = 2;
  block.Dwork(1).DatatypeID      = 0;
  block.Dwork(1).Complexity      = 'Real';
  block.Dwork(1).UsedAsDiscState = true;

  
%endfunction

function InitConditions(block)

  %% Initialize Dwork
  block.Dwork(1).Data = [0; 0];
%endfunction

function Output(block)

  block.OutputPort(1).Data = block.Dwork(1).Data;
  
%endfunction

function Update(block)
     r = block.DialogPrm(1).Data;
     h = block.DialogPrm(2).Data;
     T = block.DialogPrm(3).Data;
     u =  block.InputPort(1).Data;
     x = block.Dwork(1).Data;
     block.Dwork(1).Data=[x(1)+T*x(2); x(2)+T*fst2(x,u,r,h)];
  
%endfunction

function f=fst2(x,u,r,h)
     delta=r*h; delta0=delta*h; y=x(1)-u+h*x(2);
     a0=sqrt(delta*delta+8*r*abs(y));
     if abs(y)<=delta0,    a=x(2)+y/h;
     else,    a=x(2)+0.5*(a0-delta)*sign(y); end
     if abs(a)<=delta,    f=-r*a/delta; else,    f=-r*sign(a); end

