function varargout = pid_optimizer(varargin)
% PID_OPTIMIZER M-file for pid_optimizer.fig
% PID optimizer is a MATLAB based interface which optimizes 
% PID parameters for the plant model defined by the user.  
% In the package, Simulink is used in modelling the plant, 
% which can be SISO continuous/discrete/hybrid, and linear/nonlinear 
% of any complexity.  The optimization criteria are ITAE, ISE, 
% IAE and others, with a variety of optimization problem solvers.
%
% Type pid_optimizer at MATLAB prompt to start the program
% 
% For more information, type
%     web(['file://'  which('pid_optimizer.htm')])

% Last Modified by GUIDE v2.5 03-Jun-2008 16:09:21
% Authors: Dingyu Xue and Wenbin Dong
%          Northeastern University, Shenyang 110004, P R China
%          email: xuedingyu@mail.neu.edu.cn 
% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @pid_optimizer_OpeningFcn, ...
                   'gui_OutputFcn',  @pid_optimizer_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end
if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

function pid_optimizer_OpeningFcn(hObject, eventdata, handles, varargin)
handles.output = hObject;
guidata(hObject, handles);

% Initialization process
warning('OFF','Simulink:SL_CloseBlockDiagramNotLoaded');
load_system('cont_PID_opt');
assignin('base','u_step',1);
assignin('base','N',0.01);
assignin('base','T',0.1);
assignin('base','um',-5);
assignin('base','uM',5);
assignin('base','my_criterion',1);
assignin('base','Kp',1);
assignin('base','Ki',0);
assignin('base','Kd',0);
assignin('base','output_max',3);
set([handles.h_txtpidsample,handles.h_edtpidsample],'Visible','off');
set([handles.h_btnView,handles.h_btnOptimize,handles.h_btnMore,...
    handles.h_btnCreate,handles.h_btnOptimize,handles.h_btnsVisualize,...
    handles.h_btnShow,handles.h_btnView,handles.h_btnTerminate],'Enable','off');
u.Lower=[];
u.Upper=[];
u.saturation='-5,5';
u.final_value='1';
u.ckstiff=0;
u.ckfix=0;
u.step=[];
u.filter='0.01';
u.sample=[];
u.overshoot=[];
u.tool=2;
u.h_main=handles.h_Main;
u.h_more=[];
u.optf=[];
u.cons=[];
u.tend=[];
u.vars=[];
u.nvars=[];
u.plant=[];
u.pidmdl='cont_PID_opt';
u.handles=handles;
set(handles.h_Main,'UserData',u);

function varargout = pid_optimizer_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;

% This function receives the plant model name from the user
% and embed the plant model into the simulation model of the
% closed-loop system with PID-type controller.  Inports and
% outports are examined to ensure the plant is a SISO one.
function h_edtFile_Callback(hObject, eventdata, handles)
u=get(handles.h_Main,'UserData');
FileN=get(handles.h_edtFile,'String');
if exist(FileN,'file')~=4
    errordlg('The selected Simulink file is not in the search path.',...
        'Error: Model problem');
else
    if length(FileN)>3 && FileN(end-3)=='.'
        u.plant=FileN(1:end-4);
    else
        u.plant=FileN;
    end
    load_system(u.plant)
    inputnum=find_system(u.plant,'BlockType','Inport');
    outputnum=find_system(u.plant,'BlockType','Outport');
    if length(inputnum)~=1 || length(outputnum)~=1 % if not SISO
        errordlg('Wrong Simulink model, the must must have only one In and Out.','Error: Simulink file');
    else
        load_system(u.pidmdl);
        set_param([u.pidmdl '/Embeded Plant Model'],'ModelName',u.plant);
        set_param(u.pidmdl,'SolverPrmCheckMsg','none');
        set([handles.h_btnMore,handles.h_btnCreate,handles.h_btnsVisualize,handles.h_btnShow],'Enable','on');
        set(handles.h_edtFile,'String',u.plant)
        set(handles.h_Main,'UserData',u);
        save_system(u.pidmdl)
    end
end
set(handles.h_btnOptimize,'Enable','off');
set(handles.h_btnTerminate,'Enable','off','String','Pause');

% This function allows the user to select an existing plant
% Simulink model with a common file dialog box.
function h_btnBrowse_Callback(hObject, eventdata, handles)
[FileN,PathN]=uigetfile('*.mdl','Select a model name');
set(handles.h_edtFile,'String',FileN);
h_edtFile_Callback(hObject, eventdata, handles);

% set criterion control constant in MATLAB workspace
function h_lstError_Callback(hObject, eventdata, handles)
assignin('base','my_criterion',get(handles.h_lstError,'Value'));

% Create automatically the objective function and constraint
% functions, if any, in M-files.
function h_btnCreate_Callback(hObject, eventdata, handles)
u=get(handles.h_Main,'UserData');
kVariables=get(handles.h_lstVariables,'value');
set(handles.h_btnTerminate,'String','Pause');
switch kVariables
    case 1
        strvars{1}='Kp';strvars{2}='Ki'; n=2;
    case 2
        strvars{1}='Kp';strvars{2}='Kd'; n=2;
    case 3
        strvars{1}='Kp';strvars{2}='Ki'; strvars{3}='Kd'; n=3;
end
kopt=0;
while 1
    str=['optfun_' int2str(kopt)];
    if ~any([2 3 5 6]==exist(str,'file')), break; end
    kopt=kopt+1;
end
con=['optcon_' int2str(kopt)];
if isempty(get(handles.h_edtTerminate,'String'))
    t_end=10; set(handles.h_edtTerminate,'String','10')
else
    t_end=str2num(get(handles.h_edtTerminate,'String'));
end
u.nvars=n;
u.vars=strvars;
u.tend=t_end;
u.plant=get(handles.h_edtFile,'String');
u.cons=con;
u.optf=str;
set(handles.h_Main,'UserData',u);
key=optfun_file_save(str,handles);
if key==1
    set([handles.h_btnView,handles.h_btnOptimize],'Enable','on');
end
set_param([u.pidmdl,'/Multi-stairs source'],'MaskValueString','[0 1]|[1 1]*u_step')
save_system(u.pidmdl)

function h_btnOptimize_Callback(hObject, eventdata, handles)
h_btnsVisualize_Callback(hObject, eventdata, handles)
set(handles.h_btnTerminate,'Enable','on','String','Pause');
u=get(handles.h_Main,'UserData');
if u.tool==4 || u.tool==5
    lb=convert_str(u.nvars,u.Lower,0);
    ub=convert_str(u.nvars,u.Upper,10);
else
    lb=convert_str(u.nvars,u.Lower,-inf);
    ub=convert_str(u.nvars,u.Upper,inf);
end
str_x0=get(handles.h_edtInit,'String');
x0=convert_str(u.nvars,str_x0);
load_system(u.plant);
%set_param(u.plant,'InlineParams','on','SolverPrmCheckMsg','none');
ff=msgbox('Optimization process on, please wait...','Wait');
if u.ckfix==1
    set_param(u.pidmdl,'FixedStep',str2num(u.step));
    set_param(u.plant, 'FixedStep',str2num(u.step));
end
if u.ckstiff==1
    set_param(u.pidmdl,'Solver','ode15s','MaxOrder','5');
    set_param(u.plant, 'Solver','ode15s','MaxOrder','5');
else
    set_param(u.pidmdl,'Solver','ode45','MaxOrder','5');
    set_param(u.plant, 'Solver','ode45','MaxOrder','5');
end
save_system(u.pidmdl); save_system(u.plant)
switch u.tool
    case 1
        options=optimset('Display','iter','Jacobian','off','LargeScale','off',...
            'OutputFcn',@outfun,'TolX',1e-3,'TolFun',1e-3,'MaxIter',50);
        if isempty(lb) && isempty(ub) && isempty(u.overshoot)
            ctrl_pars=fminsearch(u.optf,x0,options);
        elseif ~isempty(u.overshoot)
            errordlg({'Cannot solve constrained optimization Problems,use optimization Toolbox ';lasterr},'Error');
        end
    case 2
        options=optimset('Display','iter','Jacobian','off','LargeScale','off',...
                'OutputFcn',@outfun,'TolX',1e-3);
        if isempty(lb) && isempty(ub) && isempty(u.overshoot)
            ctrl_pars=fminunc(u.optf,x0,options); 
        elseif isempty(u.overshoot)
            ctrl_pars=fminsearchbnd(u.optf,x0,lb,ub,options);
        else
            ctrl_pars=fmincon(u.optf,x0,[],[],[],[],lb,ub,u.cons,options);
        end
    case 3
        options=gaoptimset('Generations',20,'HybridFcn',@fmincon,'PopulationSize',30,'StallGenLimit',100);
        if isempty(u.overshoot)                
            eval(['ctrl_pars=ga(@' u.optf ',u.nvars,[],[],[],[],lb,ub,[],options)']);
        else
            eval(['ctrl_pars=ga(@' u.optf ',u.nvars,[],[],[],[],lb,ub,@' u.cons ',options);'])
        end
    case 4
        limits=[lb(:) ub(:)];
        if isempty(limits)
            errordlg({'Input the bounds for variables'; lasterr},'Error');
        else
            ctrl_pars=gaopt(limits,u.optf,[],[],[],'maxGenTerm',30);
            ctrl_pars=ctrl_pars(1:end-1); eval([u.optf '(ctrl_pars)']);
        end
    case 5
        ctrl_pars=PSO_Trelea_vectorized(u.optf,u.nvars,4,[lb(:),ub(:)]);
end
set(handles.h_edtInit,'String',['[' num2str(ctrl_pars(:).') ']''']);
try, delete(ff), end

% display the plant model
function h_btnShow_Callback(hObject, eventdata, handles)
open_system(get(handles.h_edtFile,'String'));

% display in M-editors the objective/constraint functions
function h_btnView_Callback(hObject, eventdata, handles)
u=get(handles.h_Main,'UserData');
edit(u.optf);
if ~isempty(u.overshoot) && ~any([4 5]==u.tool), edit(u.cons); end

function h_btnTerminate_Callback(hObject, eventdata, handles)
vars=get(handles.h_btnTerminate,'String');
switch vars 
    case 'Pause'
        set(handles.h_btnTerminate,'String','Resume');
    case 'Resume'
        h_btnOptimize_Callback(hObject, eventdata, handles)
end

% exit the interface, close all the opened models
function h_btnCancel_Callback(hObject, eventdata, handles)
close(gcf);
close_system('disc_PID_opt',0)
close_system('cont_PID_opt',0)
close_system('tf_contmodel',0)
close_system('tf_discmodel',0)

% delete all the objective/constraint functions
function h_btnClear_Callback(hObject, eventdata, handles)
set([handles.h_btnOptimize,handles.h_btnView],'Enable','off');
for i=0:1000
    try
        str=['optfun_' int2str(i)];
        con=['optcon_' int2str(i)];
        if exist(str)==2, delete([str '.m']); end
        if exist(con)==2, delete([con '.m']); end
    catch
        errordlg({['Error deleting files' int2str(i)];lasterr},'Error');
    end
end

% open htm file browser to display tutorial information.
function h_btnHelp_Callback(hObject, eventdata, handles)
web(['file://' which('pid_optimizer.htm')]);

function h_btnsVisualize_Callback(hObject, eventdata, handles)
u=get(handles.h_Main,'UserData');
set_param([u.pidmdl '/Scope'],'Open','on');
save_system(u.pidmdl)

function h_raoContinuous_Callback(hObject, eventdata, handles)
u=get(handles.h_Main,'UserData');
u.pidmdl='cont_PID_opt';
load_system(u.pidmdl)
set(handles.h_Main,'UserData',u);
set_param('cont_PID_opt/Embeded Plant Model','ModelName',u.plant);
set([handles.h_txtpidsample,handles.h_edtpidsample],'Visible','off');
save_system(u.pidmdl)

function h_raoDiscrete_Callback(hObject, eventdata, handles)
u=get(handles.h_Main,'UserData');
set([handles.h_txtpidsample,handles.h_edtpidsample],'Visible','on');
set(handles.h_edtpidsample,'String',num2str(u.sample));
u.pidmdl='disc_PID_opt';
load_system(u.pidmdl)
set(handles.h_Main,'UserData',u);
set_param('disc_PID_opt/Embeded Plant Model','ModelName',u.plant);
save_system(u.pidmdl)

function key=optfun_file_save(str,handles,PathN)
u=get(handles.h_Main,'UserData');
kTool=u.tool; key=1; kstr=''; id_str='';
try
    if nargin<3, PathN=''; end
    fid=fopen([PathN str,'.m'],'w');
    if kTool==4 % if GAOT is used, rewrite the objective function
        fprintf(fid,'function [sol,y]=%s(x,options)\r\n',str); kstr='-';
    elseif kTool==5, id_str='    ';
        fprintf(fid,'function y=%s(X)\r\n',str);
    else
        fprintf(fid,'function y=%s(x)\r\n',str);
    end
    fprintf(fid,'%% %s   An objective function for optimal controller design\r\n',upper(str));
    fprintf(fid,'%%\r\n\r\n');
    fprintf(fid,'%%   The function is created by PID toolbox.\r\n');
    fprintf(fid,'%%   Date of creation %s\r\n',date);
    if kTool==4, fprintf(fid,'sol=x;\r\n');
    elseif kTool==5,
        fprintf(fid,'[m,n]=size(X); Y=[];\r\n');
        fprintf(fid,'for i=1:m, x=X(i,:);\r\n');
    end
    for i=1:u.nvars
        fprintf(fid,'%sassignin(''base'',''%s'',x(%d));\r\n',id_str,u.vars{i},i);
    end
    fprintf(fid,'%sopt=simset(''OutputVariables'',''y'');\r\n',id_str);
    fprintf(fid,'%stry\r\n',id_str);
    fprintf(fid,'   %s[t_time,x_state,y_out]=sim(''%s'',[0,%f],opt);\r\n',id_str,u.pidmdl,u.tend);
    fprintf(fid,'   %sy=%sy_out(end,1);\r\n',id_str,kstr);
    if ~isempty(u.overshoot)
        overshoot=str2num(u.final_value)*(1+str2num(u.overshoot)/100);
        if kTool==4 || kTool==5
            fprintf(fid,'   %sif (max(y_out(:,2))-%f)>0\r\n',id_str,overshoot);
            fprintf(fid,'       %sy=%s100000;\r\n   end\r\n',id_str,kstr);
            fprintf(fid,'%scatch\r\n   %sy=%s100000;\r\nend\r\n',id_str,id_str,kstr);
            if kTool==5, fprintf(fid,'        Y=[Y; y];\r\n; y=Y;\r\n    end '); end
            fclose(fid);
        else
            fprintf(fid,'   %sassignin(''base'',''output_max'',max(y_out(:,2)));\r\n',id_str);
            fprintf(fid,'%scatch\r\n   %sy=%s100000;\r\n%send\r\n',id_str,id_str,kstr,id_str);
            if kTool==5, fprintf(fid,'    Y=[Y; y];\r\n;     y=Y;\r\nend '); end
            fclose(fid);
            fid=fopen([PathN u.cons,'.m'],'w');
            fprintf(fid,'function [c,ce]=%s(x)\r\n',u.cons);
            fprintf(fid,'%% %s   An constraint function for optimal controller design\r\n',upper(u.cons));
            fprintf(fid,'%%\r\n\r\n');
            fprintf(fid,'%%   The function is created by PID toolbox.\r\n');
            fprintf(fid,'%%   Date of creation %s\r\n',date);
            fprintf(fid,'ce=[]; c=evalin(''base'',''output_max'')-%f;',overshoot);
            fclose(fid);
        end
    else
        fprintf(fid,'%scatch\r\n   %sy=%s100000;\r\n%send\r\n',id_str,id_str,kstr,id_str);
        if kTool==5, fprintf(fid,'    Y=[Y; y];\r\nend\r\ny=Y(:);'); end
        fclose(fid);
    end
catch
    key=0;
    errordlg({'Error Creating optimization objective function file';lasterr},'Error');
end

function h_btnMore_Callback(hObject, eventdata, handles)
more_specification;

% Linear transfer function input
function h_raolinear_Callback(hObject, eventdata, handles)
linear_model; 
set(handles.h_raononlinear,'Value',1)
set(handles.h_raolinear,'Value',0)

% get ready to accept nonlinear Simulink model
function h_raononlinear_Callback(hObject, eventdata, handles)
set(handles.h_edtFile,'String','');

% user termination of optimization process
function stop = outfun(x,optimValues,state)
stop = false;
hMain=findobj('Tag','h_Main');
u=get(hMain,'UserData');
% Check if user has requested to stop the optimization.
strings=get(u.handles.h_btnTerminate,'String');
switch strings
    case 'Pause'
        stop=false;        
    case 'Resume'
        stop=true;        
        setappdata(u.handles.h_btnOptimize,'ctrl_pars',x);
end

function arr_num = convert_str(n,str_input,flag)
if isempty(str_input) 
    if nargin==2, arr_num=rand(n,1);
    else, arr_num=[]; end
else
    if nargin==2, flag=1; end
    arr_num0=str2num(str_input); [k,m]=size(arr_num0);
    if m*k==1, arr_num=arr_num0*ones(n,1);
    elseif k<n, arr_num=[arr_num0(:); flag*ones(n-k,1)];
    else, arr_num=arr_num0(1:n);
    end
end

function h_Main_CloseRequestFcn(hObject, eventdata, handles)
h_btnCancel_Callback(hObject, eventdata, handles)

function h_edtpidsample_Callback(hObject, eventdata, handles)
u=get(handles.h_Main,'UserData');
try
    u.sample=str2num(get(handles.h_edtpidsample,'String'));
    set(handles.h_Main,'UserData',u);
    assignin('base','T',u.sample);
catch
    f = errordlg('Wrong sampling period. Try again!.', 'Error', 'modal');
end

function h_btnMultiStep_Callback(hObject, eventdata, handles)
more_simulate

% The following functions are empty ones, which should not be modified.
function h_Main_CreateFcn(hObject, eventdata, handles)
function h_txtpidsample_CreateFcn(hObject, eventdata, handles)
function h_btnBrowse_CreateFcn(hObject, eventdata, handles)
function text2_CreateFcn(hObject, eventdata, handles)
function text3_CreateFcn(hObject, eventdata, handles)
function h_edtInit_Callback(hObject, eventdata, handles)
function h_edtTerminate_Callback(hObject, eventdata, handles)
function h_edtpidsample_CreateFcn(hObject, eventdata, handles)
function h_lstVariables_Callback(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
function h_edtTerminate_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
function h_edtInit_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
function h_lstVariables_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
function h_edtFile_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
function h_lstError_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
